@@ -6,7 +6,7 @@ import time
 from flask import Flask, request
 from tinydb import TinyDB, where
 from PM3.model.process import Process
-from PM3.model.pm3_protocol import RetMsg
+from PM3.model.pm3_protocol import RetMsg, KillMsg
 import logging
 from collections import namedtuple
 from configparser import ConfigParser
@@ -16,6 +16,7 @@ from pathlib import Path
 from PM3.libs.pm3table import Pm3Table, ION
 import signal
 import json
+import threading
 
 pm3_home_dir = os.path.expanduser('~/.pm3')
 config_file = f'{pm3_home_dir}/config.ini'
@@ -31,14 +32,14 @@ db = TinyDB(config['main_section'].get('pm3_db'))
 tbl = db.table(config['main_section'].get('pm3_db_process_table'))
 ptbl = Pm3Table(tbl)
 
-#from flask_crontab import Crontab
-
 app = Flask(__name__)
-#crontab = Crontab(app)
 ret_msg = namedtuple('RetMsg', 'msg, err')
+alive_gone = namedtuple('AliveGone', 'pid')
 
-#TODO: refactoring... e' un dizionario!
-process_running_list = {}
+# Processi avviati localmente con popen:
+# key = pid
+# value = processo Popen
+local_popen_process = {}
 
 def _resp(res: RetMsg) -> dict:
     if res.err:
@@ -72,12 +73,13 @@ def _start_process(proc, ion) -> RetMsg:
         msg = f'process {proc.pm3_name} (id={proc.pm3_id}) already running with pid {proc.pid}'
         return RetMsg(msg=msg, warn=True)
     elif proc.restart >= proc.max_restart:
+        # Max request exceded
         msg = f'ERROR, process {proc.pm3_name} (id={proc.pm3_id}) exceded max_restart {proc.restart}/{proc.max_restart}'
         return RetMsg(msg=msg, err=True)
     else:
         try:
             p = proc.run()
-            process_running_list[proc.pid] = p
+            local_popen_process[proc.pid] = p
             if not ptbl.update(proc):
                 # Update Error
                 msg = f'Error updating {proc}'
@@ -134,6 +136,31 @@ def new_process():
         return _resp(RetMsg(msg=msg, err=True))
 
 
+def _local_kill(proc):
+    p = local_popen_process[proc.pid]
+    local_pid = p.pid
+    p.kill()
+    for i in range(5):
+        _ = p.poll()
+        if not proc.is_running:
+            break
+        time.sleep(1)
+    else:
+        return KillMsg(msg='OK', alive=[alive_gone(pid=local_pid),], warn=True)
+    return KillMsg(msg='OK', gone=[alive_gone(pid=local_pid),])
+
+def _interal_poll():
+    for local_pid, p in local_popen_process.items():
+        p.poll()
+
+def _interal_poll_thread():
+    # Interrogazione ciclica dei processi avviati da PM3
+    # i processi contenuti in local_popen_process
+    # vanno periodicamente interrogati
+    while True:
+        _interal_poll()
+        time.sleep(1)
+
 @app.get("/stop/<id_or_name>")
 @app.get("/restart/<id_or_name>")
 @app.get("/rm/<id_or_name>")
@@ -145,18 +172,25 @@ def stop_and_rm_process(id_or_name):
         resp_list.append(_resp(RetMsg(msg=msg, err=True)))
 
     for proc in ion.proc:
-        ret = proc.kill()
+        if proc.pid in local_popen_process:
+            # Processi attivati da os.getpid() vanno trattati con popen
+            ret = _local_kill(proc)
+        else:
+            ret = proc.kill()
+
         if ret.msg == 'OK':
+            proc.autorun_exclude = True
             ptbl.update(proc)
             for pk in ret.gone:
                 msg = f'process {proc.pm3_name} (id={proc.pm3_id}) with pid {pk.pid} was killed'
                 resp_list.append(_resp(RetMsg(msg=msg, err=False)))
+        elif ret.warn:
             for pk in ret.alive:
                 msg = f'process {proc.pm3_name} (id={proc.pm3_id}) with pid {pk.pid} still alive'
                 resp_list.append(_resp(RetMsg(msg=msg, warn=True)))
-        elif ret.warn:
-            msg = f'process {proc.pm3_name} (id={proc.pm3_id}) not running'
-            resp_list.append(_resp(RetMsg(msg=msg, warn=True)))
+            if ret.alive == 0:
+                msg = f'process {proc.pm3_name} (id={proc.pm3_id}) not running'
+                resp_list.append(_resp(RetMsg(msg=msg, warn=True)))
         else:
             msg = f'Strange Error'
             resp_list.append(_resp(RetMsg(msg=msg, warn=True)))
@@ -179,10 +213,8 @@ def ls_process(id_or_name):
     payload = []
     ion = ptbl.find_id_or_name(id_or_name)
     for proc in ion.proc:
-
-        # NON CAPISCO A CHE SERVE, ELIMINARE
-        #if proc.pid in process_running_list:
-        #    process_running_list[proc.pid].poll()
+        # Aggiorno lo stato dei processi
+        _interal_poll()
         # Trick for update pid
         proc.is_running
         ptbl.update(proc)
@@ -249,6 +281,7 @@ def _make_fake_backend(pid, cwd):
                    stderr=f'{pm3_home_dir}/__backend__.err',
                    pid=pid,
                    cwd=cwd,
+                   restart=1,
                    max_restart=100000)
     return proc
 
@@ -285,7 +318,7 @@ def main():
         proc_backend.cwd = my_cwd
         proc_backend.is_running
         ptbl.update(proc_backend)
-    process_running_list[proc_backend.pid] = proc_backend
+
 
     # __cron_checker__ process
     ion_cron = ptbl.find_id_or_name('__cron_checker__')
@@ -307,6 +340,10 @@ def main():
         if ret_m['err'] is True:
             print(ret_m)
 
+    # Threads
+    t1 = threading.Thread(target=_interal_poll_thread)
+    t1.start()
+
     print(f'running on pid: {my_pid}')
     app.run(debug=True, host=dsn.host, port=dsn.port)
 
