@@ -7,13 +7,15 @@ import os
 from pathlib import Path
 import pendulum
 import signal
+from PM3.model.pm3_protocol import KillMsg
+from collections import namedtuple
+
+alive_gone = namedtuple('AliveGone', 'pid')
+
+def on_terminate(proc):
+    print(proc.status())
+    print("process {} terminated with exit code {}".format(proc.pid, proc.returncode))
 
-class KillMsg(BaseModel):
-    msg: str = ''
-    err: bool = False
-    warn: bool = False
-    gone: list = []
-    alive: list = []
 
 class ProcessStatusLight(BaseModel):
     pm3_id: int
@@ -182,7 +184,7 @@ class Process(BaseModel):
 
     @staticmethod
     def kill_proc_tree(pid, sig=signal.SIGTERM, include_parent=True,
-                       timeout=None, on_terminate=None):
+                       timeout=5, on_terminate=on_terminate):
         """Kill a process tree (including grandchildren) with signal
         "sig" and return a (gone, still_alive) tuple.
         "on_terminate", if specified, is a callback function which is
@@ -199,8 +201,13 @@ class Process(BaseModel):
                 p.send_signal(sig)
             except psutil.NoSuchProcess:
                 pass
-        gone, alive = psutil.wait_procs(children, timeout=timeout,
-                                        callback=on_terminate)
+        try:
+            gone, alive = psutil.wait_procs(children, timeout=timeout,
+                                            callback=on_terminate)
+        except psutil.NoSuchProcess:
+            gone = [alive_gone(pid=pid),]
+            alive = []
+
         return (gone, alive)
 
     def kill(self):
@@ -212,7 +219,6 @@ class Process(BaseModel):
             return KillMsg(msg='NO SUCH PROCESS', warn=True)
 
         gone, alive = self.kill_proc_tree(self.pid)
-        self.autorun_exclude = True
         if len(alive) > 0:
             return KillMsg(msg='OK', alive=alive, gone=gone, warn=True)
         else:
